#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Fri Feb 25 13:40:52 2022

@author: peter.dieterich@tu-dresden.de

    usage: python3 -u B_BayesianAnalysis.py model Ntra L chaintag
    
    This program takes Ntra paths from simulatedPaths_data.xlsx 
    up to L points per path. The nested sampling algorithm is
    performed for model = fBm|fBm+exp|fBm+pl. Results can be 
    found in the directory "chains-"+chaintag.

"""


import scipy
import scipy.stats
import numpy as np


import pandas as pd
from myCov import covMatTemperedFBM as covTFBM


import os, sys
import pymultinest


#
# defintion of priors of parameters
#
def myprior(cube, ndim, nparams):
    #
    cube[0] = 0.01 + 0.98 * cube[0]  		# constant prior for H
    DH0     = 50.0
    cube[1] = DH0 * np.log(1./(1.-cube[1]))	# exponential prior for DH
    cube[2] = -25.0 +  50.0 * cube[2]      # constand prior for vd

    if ndim > 3:				    # if tempering exp|powerlaw
       cube[3] = cube[3]*30			# tau*

    if ndim > 4:				    # if powerlaw tempering
       cube[4] = cube[4]*4 			# mu for power-law tempering
    #
    return


#
# definition of the log likelihood for the corresponding model
#
def myloglike(cube, ndim, nparams):
    #
    H    = cube[0]
    DH   = cube[1]
    vd   = cube[2]

    tauS = 1.0
    mu   = 0.0

    if ndim > 3: 
       tauS = cube[3]
    if ndim > 4:
       mu = cube[4]

    if model == "fBm":
        covF = covTFBM(dt=dt_0, DH=DH, H=H, L=L, tauS=1.0,  mu=0.0, tag="powerlaw")
    if model == "fBm+exp":
        covF = covTFBM(dt=dt_0, DH=DH, H=H, L=L, tauS=tauS, mu=0.0, tag="exp")
    if model == "fBm+pl":
        covF = covTFBM(dt=dt_0, DH=DH, H=H, L=L, tauS=tauS, mu=mu, tag="powerlaw")
 	    
    uu, ss, vt = scipy.linalg.svd(covF, full_matrices=True)
    invS = np.diag(1.0/ss)
    invF = np.dot(np.dot(uu,invS),vt)

    logLHsvd = 0.0
    
    for k in range(Ntra):
        a2 = - 0.5 * np.dot((xxx[k]-vd*tt), np.dot(invF, (xxx[k]-vd*tt)) )
        b2 = - 0.5 * np.sum(np.log(ss))
        c2 = - 0.5 * L * np.log(2*np.pi)
        logLHsvd += a2 + b2 + c2  

    return logLHsvd



'''
    usage: see above
                     
'''
model       = sys.argv[1]           # fBm | fBm+exp | fBm+pl
Ntra        = int(sys.argv[2])      # number of paths
L           = int(sys.argv[3])      # number of data points L < 1000
chaintag    = sys.argv[4]           # prefix of chain e. g. data-fBm-exptemp
Nlp         = 200                   # number of nested sampling points
print()
print(model, Ntra, L, Nlp, chaintag)
print()

#
# 1. read simulated or experimental data from excel file "simulatedPaths_data.xlsx"
#
excelDataFile = "simulatedPaths_data.xlsx"
df = pd.read_excel(excelDataFile, sheet_name='paths')
tt  = df.iloc[:,1].to_numpy()[:L]
xxx = np.zeros([Ntra,L])
for k in range(Ntra):
    xxx[k] = df.iloc[:,k+2].to_numpy()[:L]
dt_0 = tt[1]-tt[0]


#
# 2. model selection
#
modelfound = False
#
if model == "fBm":
    parameters = ["H", "DH", "vd"]
    modelfound = True
if model == "fBm+exp":
    parameters = ["H", "DH", "vd", "tauS"]
    modelfound = True
if model == "fBm+pl":
    parameters = ["H", "DH", "vd", "tauS", "mu"]
    modelfound = True

if modelfound == False:
    print("model not found --> EXIT")
    sys.exit(0)
n_params = len(parameters)
print(n_params, parameters)


#
# 3. create chains directory
#
chainsName = "chains-"+chaintag
if not os.path.exists(chainsName): os.mkdir(chainsName)


#
# 4. start nested sampling process
#
pymultinest.run(myloglike, myprior, n_params, importance_nested_sampling = False, resume = False, verbose = True, sampling_efficiency = 'parameter', 
                n_live_points = Nlp, init_MPI=False, outputfiles_basename = chainsName+"/1-")
