#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jan 15 11:21:47 2022

@author: peter.dieterich@tu-dresden.de

"""

import scipy
import scipy.special as sc
import numpy as np



########################################################################################
#
#  <x(t) x(t')> MATRIX OF TEMPERED FRACTIONAL BROWNIAN MOTION 
#               (also see Molina-Garcia 2018 NJP)
#
#   - <x(t) x(t')> calculated analytically in Dieterich et al. "Analysis of anomalous 
#     diffusion and tempering in neutrophil chemotaxis applying Bayesian inference and 
#     covariances" (2022).
#
#   - fBm with powerlaw or exponential tempering.
#
#   - fBm is obtained with mu=0.0 and tag="powerlaw".
#
#
#   dt      sampling time interval
#   DH      generalized diffusion coefficient
#   H       Hurst coefficient (0<H<1)
#   L       number of simulated points
#   tauS    tempering parameter
#   mu      tempering coefficient
#   tag     powerlaw|exp    
#
########################################################################################
def covMatTemperedFBM(dt=1, DH=1.0, H=0.5, L=1, tauS=1.0, mu=0.0, tag="powerlaw"):
    G = scipy.reshape(scipy.zeros(L**2), (L, L))
    #
    if tag=="exp":
        pref = DH/sc.gamma(2.*H+1.); al=2.*H 
        F11  = np.zeros(L)
        for i in range(L):
            ti = (i+1)*dt/tauS
            F11[i] = sc.hyp1f1(2.*H-1.0, 2.*H+1.0, -ti)
        for i in range(L): # t
            ti = dt*(i+1)
            for j in range(i+1): # t'
                tj = dt*(j+1)
                G[i,j] = pref*( ti**al * F11[i] + tj**al * F11[j] - abs(ti-tj)**al * F11[abs(i-j)] )
        for i in range(L):
            for j in range(i):
                G[j,i] = G[i,j]
    #
    if tag=="powerlaw":
        pref = DH/sc.gamma(2.*H+1.); al=2.*H 
        F21  = np.zeros(L)
        for i in range(L):
            ti = (i+1)*dt/tauS
            F21[i] = sc.hyp2f1(mu, 2.*H-1.0, 2.*H+1.0, -ti)
        for i in range(L): # t
            ti = dt*(i+1)
            for j in range(i+1): # t'
                tj = dt*(j+1)
                G[i,j] = pref*( ti**al * F21[i] + tj**al * F21[j] - abs(ti-tj)**al * F21[abs(i-j)] )
        for i in range(L):
            for j in range(i):
                G[j,i] = G[i,j]
    #
    return G



########################################################################################
#
# <x(t) x(t')> MATRIX OF THE ORNSTEIN UHLENBECK PROCESS
#
#   dt      sampling time interval
#   v2      mean squared velocity
#   ga      persistence parameter
#   L       number of simulated points
#
########################################################################################        
def covOU(dt=1.0, v2=1.0, ga=1.0, L=1):
    G = np.reshape(np.zeros(L**2), (L, L))
    for i in range(L): # t
        ti = dt*(i+1)
        for j in range(i+1): # t'
            tj = dt*(j+1)
            G[i,j] = v2/ga**2*( 2*tj*ga - 1.0 - np.exp(-ga*abs(ti-tj))  + np.exp(-ga*ti) + np.exp(-ga*tj) )
    for i in range(L):
        for j in range(i):
            G[j,i] = G[i,j]

    return G
